from django.conf.urls import url, include

from otree_tools import views as v


urlpatterns = [url(v.ListPVarsView.url_pattern, v.ListPVarsView.as_view(),
                   name=v.ListPVarsView.url_name),
               url(v.PVarsCSVExport.url_pattern, v.PVarsCSVExport.as_view(),
                   name=v.PVarsCSVExport.url_name),
               ]



from otree.models_concrete import PageCompletion


def get_seconds_per_page(player, page_name):
    participant = player.participant
    subsession_pk = player.subsession.pk
    try:
        page_info = PageCompletion.objects.get(participant=participant, subsession_pk=subsession_pk,
                                               page_name=page_name)
    except PageCompletion.DoesNotExist:
        raise ValueError('Cannot find info about this page')
    return page_info.seconds_on_page


from otree_tools.models import EnterEvent
from django.db.models import ExpressionWrapper, F, DurationField
from django.db.models import Count, Min, Sum


def get_time_per_page(player, page_name):
    tot_enter_events = EnterEvent.objects.filter(participant=player.participant,
                                                 page_name=page_name
                                                 ).annotate(num_exits=Count('exits')).filter(num_exits__gt=0)
    if tot_enter_events.exists():
        b = tot_enter_events.annotate(
            early_exits=Min('exits__timestamp'),
            timediff=ExpressionWrapper(F('early_exits') - F('timestamp'),
                                       output_field=DurationField())
        ).aggregate(sum_diff=Sum('timediff'))

        sum_diff = b['sum_diff']

        return sum_diff



from django import template
from django.template import TemplateSyntaxError, Context


class ButtonTagError(Exception):
    pass


register = template.Library()
# two following random strings are needed to prevent usage of tracking_time and tracking_focus twice
# see more here: https://stackoverflow.com/questions/51786795/check-that-tag-is-used-in-template-only-once/
tracking_time_code, tracking_focus_code ='xBl4fTBP8hAsg61o1gJa','Q4YviMPgcVUjbbm05C6s'


def universal_tracker(context, tagname):
    context['page_index'] = context['view']._index_in_pages
    context['page_name'] = context['view'].__class__.__name__
    code =globals()['{}_code'.format(tagname)]
    if code in context:
        formatted_tag_name = '{{% {} %}}'.format(tagname)
        raise Exception('{} is already used'.format(formatted_tag_name))
    context[code] = True
    return context

@register.inclusion_tag('otree_tools/tags/TimeTracker.html', takes_context=True)
def tracking_time(context, *args, **kwargs):
    return universal_tracker(context, 'tracking_time')


@register.inclusion_tag('otree_tools/tags/FocusTracker.html', takes_context=True, name='tracking_focus')
def tracking_focus_func(context, *args, **kwargs):

    return universal_tracker(context, 'tracking_focus')


@register.inclusion_tag('otree_tools/tags/Button.html', takes_context=True)
def button(context, label='', *args, **kwargs):
    for kwarg in kwargs:
        raise ButtonTagError(
            # need double {{ to escape because of .format()
            '{{% button %}} tag received unrecognized parameter "{}"'.format(kwarg)
        )
    context['label'] = label
    return context






# TODO: incorporate ~admin to include mturk, paypal and json export


from otree_tools.views import EnterExitEventList, FocusEventList, PVarsSessionsList
data_export_views = [EnterExitEventList, FocusEventList,PVarsSessionsList]
# TODO: export participant vars export
